      SUBROUTINE SHEBA_BULK(TAU,HS,HL,EVAP)
C  Last modification 10/14/14.
C  Version 2.0.
C  This program implements the SHEBA Bulk Flux Algorithm.
C  It makes bulk estimates of USTAR and the sensible and latent heat
C  fluxes using paraterizations that are consistent with SHEBA data.
C  For roughness length z0, the program uses our new drifting snow
C  algorithm for winter (judged by an ice concentration .GE. 0.995)
C  and uses a prediction for CDN10 based on ice concentration for
C  summer.
C  This version also allows specifying the physical roughness (XI),
C  and then uses the Andreas (2011) algorithm to calculate Z0.
C  For the scalar roughness lengths, the program uses Andreas's (1987)
C  model.  The wind speed also includes the COARE gustiness factor in
C  unstable stratification and the windless coefficient of Jordan et al.
C  (1999) in stable stratification.
C  RU, RT, and RQ are measurement heights for wind speed, temperature, and humidity.
C  UR is the wind speed.
C  DEL_THETA is the surface-air difference in potential temperature.
C  DEL_Q is the surface-air difference in specific humidity.
C  T_AVE is the average temperature between the surface and RT.
C  Q_AVE is the average specific humidity between the surface and RQ.
C  L ia the estimated Obukhov length.
C  USTAR is the estimated friction velocity in m/s.
C  HS is estimated sensible heat flux in W/m^2.
C  HL is the estimated latent heat flux in W/m^2.
C  ZO, ZT, and ZQ are the roughness lengths.
C  ITS is the number of iterations to reach a solution.
C
C
      USE CONSTANTS                   !Specifies K and G
	USE CONDITIONS                  !For passing input conditions
	USE RESULTS                     !For passing results
C
      IMPLICIT NONE
C
C       Passed variables.
      REAL TAU, HS, HL, EVAP
C
      INTEGER ICE_CODE, KODE
C       ICE_CODE is code for winter or summer ice.
C       KODE is the code for stability in the iteration.
C
C       Computed conditions.
      REAL THETAR, DEL_THETA, TAVE, DEL_Q, QAVE, RHS, RHOA, QS, F,
     A ESAT_W, ESAT_I
C       THETAR is the potential temperature at height RT.
C       DEL_THETA is the surface-air difference in potential temperature.
C       TAVE is the "aveage" temperature of the surface layer.
C       DEL_Q is the surface-air difference in specific humidity.
C       QAVE is the "average" specific humidity of the surface layer.
C       RHS (= 1.0) is the fractional relative humidity of the surface.
C       QS is the specific humidity of the surface.
C       F is the fractional relative humidity at height RT.
C       ESAT_W is the saturation vapor pressure with respect to water.
C       ESAT_I is the saturation vapor pressure with respect to ice.
C
C       Other computed conditions directly leated to the fluxes.
      REAL GNU, CP, LV
C       GNU is the kinematic viscosity of air.
C       CP is the specific heat of air at constant pressure.
C       LV is the latent heat of sublimation.
C
C       Quantities used in the iteration to find the fluxes.
      REAL CDN10, CDN, CD, CH, CE, UR0, USTAR_OLD, TSTAR_OLD,
     A QSTAR_OLD, TESTU, TESTT, TESTQ
C       CDN10 is the neutral-stability drag coefficient at 10 m.
C       CDN is the first estimate of CD.
C       CD is the drag coefficient.
C       CH is the transfer coefficient for sensible heat.
C       CE is the transfer coefficient for latent heat.
C       UR0 is a saved value of UR.
C       USTAR_OLD	is the previous estimate of USTAR.
C       TSTAR_OLD is the previous estimate of TSTAR.
C       QSTAR_OlD is the previous estimate of QSTAR.
C       TESTU is the convergence test value for USTAR.
C       TESTT is the convergence test value for TSTAR.
C       TESTQ is the convergence test value for QSTAR.
C
C       Miscellaneous.
      REAL DUM, DUM1
C
C       External real functions.
      REAL, EXTERNAL :: NU, LATENT
C	  NU is a routine that calculates kinematic viscosity.
C	  LATENT is a function that computes LV.
C
C       External functions.
      REAL, EXTERNAL :: SPEC_HT_AIR, ROUGHNESS, XI_ROUGHNESS, OBUKHOV,
     A CDNR, SPEED
C       SPEC_HEAT_AIR finds the specific heat of air.
C       ROUGHNESS calculates the roughness length from CDNR.
C       XI_ROUGHNESS calculates the roughness length from the physical
C       roughness XI.
C       OBUKHOV calculates the Obukhov length.
C       CDNR calculates the neutral-stability drag coefficient at height UR.
C       SPEED modifies the given wind speed for gustiness or windless transfer.
C
C       External subroutines.
      EXTERNAL DEN_FROM_RH, SATVPR_BZ, CD_ICE_WATER, BULK_COEFS,
     A SHEBA_WINTER_Z0, ANDREAS_87, FLUXES
C	  DEN_FROM_RH calculates the air density and water vapor variables.
C       SATVPR_BZ calculates the saturation vapor pressure.
C       CD_ICE_WATER is the SHEBA roughness algorithm for summer and the
C       marginal ice zone.
C       BULK_COEFS calculates the stability-corrected transfer coefficients.
C       SHEBA_WINTER_Z0 calculates the roughness length over winter sea ice.
C       ANDREAS_87 calculates the scalar roughness lengths, ZT and ZQ.
C       FLUXES calculates the fluxes after the iteration finds USTAR, TSTAR,
C       and QSTAR.
C
C
      REAL, PARAMETER :: ZI = 600.0, Z0_GUESS = 2.30E-4, CONV = 0.001,
     A R10 = 10.0
C	  ZI is a guess as to the inversion height.
C	  Z0_GUESS is the first guess as to Z0 for the SHEBA drag algorithm.
C	  CONV is the convergence limit for iterations on USTAR, TSTAR,
C	  and QSTAR.
C	  R10 is the 10-m reference height necessary in the summer drag relation.
C
      INTEGER, PARAMETER :: IFLAG = 1, IFLAG_2 = 2, IFLAG_S = 2
C	  IFLAG = 1 means to always compute saturation with respect to water from
C	  input relative humidity.
C	  IFLAG_2 = 2 means to compute saturation over ice.
C	  IFLAG_S = 2 means to compute surface saturation with respect to ice.
C
C
C	  Compute necessary meteorological variables.
      THETAR = TR +(G/SPEC_HT_AIR(TR))*RT
C									!Potential temperature
      DEL_THETA = TS - THETAR			!Temperature difference
      TAVE = 0.5*(TS + TR)			!Layer-averaged temperature
C
      RHS = 1.0						!Surface at ice saturation
      CALL DEN_FROM_RH(RHS,P,TS,S,RHOA,DUM,QS,IFLAG_S)
C	  Returns density of moist air at surface (RHOA) and specific
C	  humidity at surface (QS).
      F = RH/100.0					!Fractional relative humidity
      CALL DEN_FROM_RH(F,P,TR,S,DUM,DUM1,QR,IFLAG)
C	  Returns specific humidity at height RQ.
      DEL_Q = QS - QR					!Humidity difference
      QAVE = 0.5*(QS + QR)			!Layer-averaged Q
C	  Find relative humidity with respect to ice from reported RH wrt water.
      IF(TR .LE. 0.0) GO TO 30
C	  Otherwise, makes no sense to compute saturation over ice.
      RH_I = 999.99					!Flag meaningless value
	GO TO 40
30    CALL SATVPR_BZ(P,TR,ESAT_W,IFLAG)
									!ESAT_W is vapor pressure in air for water sat
	CALL SATVPR_BZ(P,TR,ESAT_I,IFLAG_2)
									!ESAT_I is vapor pressure with respect to ice
	RH_I = RH*(ESAT_W/ESAT_I)		!Reported relative humidity now with respect to ice
C
C
C	  Decide on roughness model.
40    ROUGHNESS_MODEL: SELECT CASE (CHOICE)
	CASE (1)                        !Model based on XI
	ICE_CODE = 1                    !To avoid allowing this to float
	Z0 = XI_ROUGHNESS(XI)           !Z0 from XI
	CASE (2)                        !Determine Z0 from ice concentration
      ICE_CODE = 1					!Code for compact (winter) sea ice
	IF(CONC .LT. 0.995) ICE_CODE = 2
									!Code for summer ice
          WINTER_SUMMER: SELECT CASE (ICE_CODE)
          CASE (1)                    !Case of compact winter ice
          Z0 = Z0_GUESS               !Very first guess.  Iterate later
          CASE (2)                    !Combination of ice and water
          CALL CD_ICE_WATER(CONC,CDN10)
C			This returns CDN10 for the given ice concentration.
          Z0 = ROUGHNESS(R10,CDN10)   !Corresponding roughness length
		END SELECT WINTER_SUMMER
      END SELECT ROUGHNESS_MODEL
C
C
C	  Compute some thermodynamic quantities.
60    GNU = NU(TS)
C	  Kinematic viscosity of air in m**2/s.
      CP = SPEC_HT_AIR(TS)
C	  Specific heat of dry air in J/kg/K.
      LV = LATENT(TS,IFLAG_S)
C	  Latent heat of vaporization in J/kg.
C
C	  First iteration.
      ITS = 1							!Count iterations
	UR0 = UR						!Preserve UR
	IF(UR .LT. 0.1) UR0 = 0.1
C	  This assignment prevents singularity on first iteration if UR = 0.
      CDN = CDNR(RU,Z0)				!First estimate of CDNR
      USTAR = UR0*SQRT(CDN)			!First estimate of USTAR
C
C       Z0 depends on one of three possible algorithms.
      IF(CHOICE .EQ. 2 .AND. ICE_CODE .EQ. 1)
     A CALL SHEBA_WINTER_Z0(USTAR,GNU,Z0)
C	  Already know Z0 if CHOICE = 1 (have set it from XI)
C       or if CHOICE = 2 and ICE_CODE = 2 (have set it from
C       ice concentration, CONC)
C	  Otherwise, invoke the SHEBA winter z0 algorithm if CHOICE = 2
C       and ICE_CODE = 1.
C
70    RSTAR = (USTAR*Z0)/GNU			!Roughness Reynolds number
C	  Get values of ZT and ZQ.
      CALL ANDREAS_87(USTAR,Z0,RSTAR,ZT,ZQ)
C	  Returns roughness lengths for temperature (ZT) and humidity (ZQ).
C
      KODE = 0						!Identifies neutral stability
      CALL BULK_COEFS(Z0,ZT,ZQ,DUM,RU,RT,RQ,CD,CH,CE,KODE)
C	  Returns first estimates of CD, CH, CE.
C
C	  Make first estimates of flux scales.
      USTAR = UR0*SQRT(CD)
      TSTAR = -(CH*UR0*DEL_THETA)/USTAR
      QSTAR = -(CE*UR0*DEL_Q)/USTAR
C	  Compute Obukhov length.
      L = OBUKHOV(USTAR,TSTAR,QSTAR,TAVE,QAVE)
      KODE = 2						!Future values use stratification
C
C	  Loop to continue iterations.
C**************************************************************************
C
      DO 200 ITS=2,20					!Try 20 iterations
C	  Save previous values for comparison.
      USTAR_OLD = USTAR
      TSTAR_OLD = TSTAR
      QSTAR_OLD = QSTAR
C
C	  Z0 routing.
      IF(CHOICE .EQ. 2 .AND. ICE_CODE .EQ. 1)
     A CALL SHEBA_WINTER_Z0(USTAR,GNU,Z0)
C	  Already know Z0 if CHOICE = 1 (have set it from XI)
C       or if CHOICE = 2 and ICE_CODE = 2 (have set it from
C       ice concentration, CONC)
C	  Otherwise, invoke the SHEBA winter z0 algorithm if CHOICE = 2
C       and ICE_CODE = 1.  It returns new Z0.
C
C
100   RSTAR = (USTAR*Z0)/GNU			!Roughness Reynolds number
C
      CALL ANDREAS_87(USTAR,Z0,RSTAR,ZT,ZQ)
C	  Provides new values of ZT and ZQ.
      CALL BULK_COEFS(Z0,ZT,ZQ,L,RU,RT,RQ,CD,CH,CE,KODE)
C	  Provides new values of CD, CH, and CE.
C
      WIND = SPEED(UR,USTAR,ZI,L)		!Speed modified for gustiness
      USTAR = WIND*SQRT(CD)
      TSTAR = -(CH*WIND*DEL_THETA)/USTAR
      QSTAR = -(CE*WIND*DEL_Q)/USTAR
      L = OBUKHOV(USTAR,TSTAR,QSTAR,TAVE,QAVE)
C
C	  Test for convergence.
      TESTU = ABS((USTAR - USTAR_OLD)/USTAR)
      TESTT = ABS((TSTAR - TSTAR_OLD)/TSTAR)
      TESTQ = ABS((QSTAR - QSTAR_OLD)/QSTAR)
C
      IF(TESTU .LT. CONV) GO TO 110	!Passed if true
      GO TO 200						!Otherwise, iterate again
110   IF(TESTT .LT. CONV) GO TO 120	!Passed if true
      GO TO 200						!Otherwise, iterate again
120   IF(TESTQ .LT. CONV) GO TO 250	!Passed if true
C	  Otherwise, iterate again.
C
200   CONTINUE
C
C**************************************************************************
C	  End of iterative loop.
C
      PRINT 810
810   FORMAT(/5X,'  End without convergence.')
C
C	  Compute the fluxes.
250   CALL FLUXES(USTAR,TSTAR,QSTAR,RHOA,QS,CP,LV,TAU,HS,HL,EVAP)
C	  Returns all the fluxes.
C
      RETURN
      END