      REAL FUNCTION NU(T)
C  Last modification 6/21/00.
C  This function computes the kinematic viscosity of air in m**2/s.
C  T is the air temperature in degrees Celsius.
C
      NU = 1.326E-5*(1.0 + T*(6.542E-3 + T*(8.301E-6 - 4.840E-9*T)))
C
      END
C
C
C
      FUNCTION SPEC_HT_AIR(T)
C  Last modification 8/23/00.
C  This function computes the specific heat of dry air in J/kg/K
C  for air temperature T in degrees C between -40 and +40 C.
C
      SPEC_HT_AIR = 1005.60 + T*(0.017211 + 0.000392*T)
C
      END
C
C
C
      REAL FUNCTION LATENT(T,IFLAG)
C  Last modification 2/19/08.
C  This function computes the latent heat of vaporization or
C  sublimation in J/kg.
C  T is the temperature in degrees Celsius.
C  IFLAG can suppress a calculation of E based on temperature.
C		If IFLAG = 0, compute E based on T.
C		If IFLAG = 1, compute saturation over water.
C		If IFLAG = 2, compute saturation over ice.
C
C
      IF(IFLAG .EQ. 0) GO TO 5		!T determines if over water or over ice
      IF(IFLAG .EQ. 1) GO TO 20		!Calculations over water
      IF(IFLAG .EQ. 2) GO TO 10		!Calculations over ice

C
5     IF(T) 10, 20, 20				!Sublimation or vaporization?
C
C	  Sublimation if T is below 0 deg C.
10    LATENT = (28.34 - 0.00149*T)*1.0E+5
      RETURN
C
C	  Vaporization if T is at or above 0 deg C.
20    LATENT = (25.00 - 0.02274*T)*1.0E+5
      RETURN
C
      END
C
C
C
      SUBROUTINE DEN_FROM_RH(RH,P,TC,S,RHOA,RHOV,Q,IFLAG)
C  Last modification 7/17/06.
C  This subroutine finds the air density and various water vapor
C  quantities.
C  Inputs are:
C  RH, the fractional relative humidity.
C  P, the barometric pressure in mb.
C  TC, the temperature in degrees Celcius.
C  S, the salinity in psu.
C  IFLAG, which tells how to evaluate the saturation vapor pressure, ESAT:
C	  If IFLAG = 0, compute ESAT based on TC.
C	  If IFLAG = 1, compute ESAT for saturation over water.
C	  If IFLAG = 2, compute ESAT for saturation over ice.
C	  If IFLAG = 3, compute ESAT for saturation over water below 0 C
C  Outputs are:
C  RHOA, the density of moist air in kg/m**3.
C  RHOV, the water vapor density in kg/m**3.
C  Q, the specific humidity in kg/kg.
C
C
      REAL MW
C
C
      DATA TK/273.15/, MW/18.0150E-3/, RGAS/8.31447/, P0/1013.25/
C	  TK is 0 degrees C in kelvins.
C	  MW is the molecular weight of water in kg/mole.
C	  RGAS is the gas constant in J/mole-K.
C	  P0 is standard pressure in mb.
C
C
      T = TC + TK						!Convert to kelvins
C
      CALL SATVPR_BZ(P,TC,ESAT,IFLAG)
C	  This computes saturation vapor pressure ESAT in hPa.
C
      E = RH*ESAT						!Actual vapor pressure
      EFAC = (1.0 - 0.000537*S)*E	!Include salinity depression
C
      RHOV = (100.0*MW*EFAC)/(RGAS*T)	!Ideal gas law
C	  RHOV is the absolute humidity in kg/m**3.
C
      RHOD = 1.2922*(TK/(TC+TK))*((P-EFAC)/P0)
C									!Density of dry air in kg/m**3
C
      RHOA = RHOD + RHOV				!Density of moist air
C
      Q = RHOV/RHOA					!Specific humidity in kg/kg
C
      RETURN
      END
C
C
C
      SUBROUTINE SATVPR_BZ(P,T,E,IFLAG)
C  Last modification 7/17/06.
C  This subroutine uses Buck's (1981) equation to find the saturation vapor
C  pressure as temperature T and atmospheric pressure P.  The equation is
C
C		E = (PA*P + PB)*E0*EXP((A*T)/(B + T)
C
C  where P and E are in mb, and T is in degrees C.
C  IFLAG can suppress a calculation of E based on temperature.
C		If IFLAG = 0, compute E based on T.
C		If IFLAG = 1, compute saturation over water.
C		If IFLAG = 2, compute saturation over ice.
C		If IFLAG = 3, computes saturation over water although T below zero
C
C  This special version of the routine also has a formula for saturation over
C  water at temperatures well below 0 C so it can convert relative humidities
C  measured well below freezing and referenced to water saturation to vapor pressure.
C
C
      DIMENSION PA(3), PB(3), E0(3), A(3), B(3)
C
      DATA PA/3.46E-6,4.18E-6,3.46E-6/, PB/1.0007,1.0003,1.0007/,
     A E0/6.1121,6.1115,6.1121/,
     1 A/17.502,22.452,17.966/, B/240.97,272.55,247.15/
C	  In each array, the first entry is for saturation with respect to
C	  water, the second for ice, the third for water below 0 C.
C
C
      K = 1							!SATURATION OVER WATER
      IF(IFLAG .EQ. 0) GO TO 70		!T DETERMINES IF OVER ICE OR WATER
      IF(IFLAG .EQ. 1) GO TO 100		!CHOOSE SATURATION OVER WATER
      IF(IFLAG .EQ. 2) GO TO 50		!CHOOSE SATURATION OVER ICE
	IF(IFLAG .EQ. 3) GO TO 90		!SATURATION OVER SUBZERO WATER
	PRINT 700, IFLAG
700   FORMAT(//5X,'IFLAG = ',I6,' IS OUTSIDE THE RANGE OF ACCEPTABLE ',
     1 'VALUES.'/5X,'WE QUIT.'//)
      STOP 'BAD IFLAG'
C
50    K = 2							!SATURATION OVER ICE
      GO TO 100
C
70    IF(T .LT. 0.0) K = 2			!SATURATION OVER ICE IF TRUE
      GO TO 100
C
90    K = 3							!SATURATION OVER WATER THOUGH BELOW 0 C
C
100   E = (PA(K)*P + PB(K))*E0(K)*EXP((A(K)*T)/(B(K) + T))
C
      RETURN
      END
C
C
C
      FUNCTION ROUGHNESS(R,CDNR)
C  Last modification 7/17/14.
C  This function computes the roughness length for momentum, Z0, given the
C  neutral-stability drag coefficient CDNR appropriate at reference
C  height R.
C
C
      USE CONSTANTS
C
C
      ROUGHNESS = R*EXP(-K/SQRT(CDNR))
C
      END
C
C
C
      FUNCTION CDNR(RU,Z0)
C  Last modification 7/17/14.
C  This function computes the neutral-stability drag coefficient
C  appropriate at height RU, given the roughness length Z0.
C
C
      USE CONSTANTS
C
C
      CDNR = ((ALOG(RU/Z0))/K)**(-2)	!Neutral drag coefficient
C
      END
C
C
C
      FUNCTION SMOOTH(USTAR,GNU)
C  Last modification 11/28/01.
C  This function computes the momentum roughness length for an
C  aerodynamically smooth surface.
C  USTAR is the friction velocity in m/s.
C  GNU is the kinematic viscosity in m**2/s.
C
C
      DATA SMOOTH_COEF/0.135/
C	  This is basically the (constant) roughness Reynolds number of an
C	  aerodynamically smooth surface.
C
C
      SMOOTH = SMOOTH_COEF*(GNU/USTAR)
C
      END
C
C
C
      SUBROUTINE SHEBA_WINTER_Z0(USTAR,GNU,Z0)
C  Last modification 7/17/14.
C  This subroutine uses a tentative new SHEBA algorithm to find z0 in winter.
C  USTAR is the friction velocity.
C  GNU is the kinematic viscosity.
C  Z0 is the desired roughness length in meters.
C
      USE CONSTANTS
C
C
      DIMENSION A(2)
C
      DATA USTAR_SCALE/13.0/, Z0_MAX/2.30E-4/
C		USTAR_SCALE sets the steepness of the tanh function.
C		Z0_MAX is the nominal maximum value of z0 for this algorithm
C
      ROUGH = Z0_MAX*((TANH(USTAR_SCALE*USTAR))**3)
C
      Z0 = SMOOTH(USTAR,GNU) + ROUGH
C
      RETURN
	END
C
C
C
      SUBROUTINE CD_ICE_WATER(CONC,CDN10)
C  Last modification 1/8/08.
C  This subroutine computes the 10-meter darg coefficient at
C  neutral stability, CDN10, for a surface that is part ice and
C  part water.  The ice concentration is CONC.
C  See Andreas et al. (2005).
C
C
      IF(CONC .LT. 0.995) GO TO 50	!OK to proceed if true
C		Otherwise, not the appropriate algorithm.
      PRINT 810, CONC
810   FORMAT(2X,'Ice concentration = ',0PF5.3,
     A ' too high for CD_ICE_WATER algorithm.')
	PAUSE
C
50    IF(CONC .GE. 0.0) GO TO 100		!OK to proceed if true
C		Otherwise, bad ice concentration value.
      PRINT 820, CONC
820   FORMAT(2X,'Ice concentration = ',0PF6.3,
     A ' is not a good value.')
	PAUSE 'CD_ICE_WATER'
C
C		Otherwise, compute CDN10.
100   CDN10 = 1.0E-3*(1.500 + CONC*(2.233 - 2.333*CONC))
C
      RETURN
	END
C
C
      REAL FUNCTION XI_ROUGHNESS(XI)
C  Last modification 7/17/14.
C  This routines calculates the roughness length over sea ice (z0)
C  from the physical roughness (XI) using the analysis pioneered
C  in Banke, Smith, and Anderson (1980 and made analytical in
C  Andreas (2011).
C  XI is in centimeters.
C  Z0 is in meters.
C
C
      XI_ROUGHNESS = EXP(11.5*TANH(XI/3.2) - 18.4)
C        This is the roughness length Z0 from this algorithm.
C
      END
C
C
C
      FUNCTION OBUKHOV(USTAR,TSTAR,QSTAR,TAVE,QAVE)
C  Last modification 7/17/14.
C  This function returns the Obukhov length in meters.
C  USTAR, TSTAR, and QSTAR are flux scales, with QSTAR a scale for
C  specific humidity.
C  TAVE is the average layer temperature in Celsius.
C  QAVE is the average layer specific humidity in kg/kg.
C
      USE CONSTANTS
C
C
      DATA TK/273.15/
C
      OBUKHOV = (((TAVE+TK)*(USTAR**2))/(K*G))/
     A (TSTAR + ((0.61*(TAVE+TK)*QSTAR)/(1.0 + (0.61*QAVE))))
C
      END
C
C
C
      FUNCTION PSI_M(ZU,L)
C  Last modification 11/20/08.
C  This function computes the stability correction for the wind speed
C  profile based on new Psi functions reported by Grachev et al. (2007)
C  for stable stratification.
C  ZU is the height of the wind speed observation.
C  L is the Obukhov length.
C
      REAL L
C
      DATA BM/0.66943/
C	  BM is a coefficient from Grachev et al. (2007).
C
      ZETA = ZU/L						!Stability parameter
      IF(ZETA) 10, 30, 20				!Check stability
C
C	  Unstable stratification (Paulson, 1970).
10    X = (1 - 16.0*ZETA)**0.25
      PSI_M = 2.0*ALOG(0.5*(1.0 + X)) + ALOG(0.5*(1 + X*X))
     A - 2.0*ATAN(X) + 1.570796
      RETURN
C
C	  Stable stratification (Grachev et al. 2007).
20    X = (1.0 + ZETA)**0.333333
      TERM1 = -19.5*(X - 1.0)
	TERM2 = 2.17566*(2.0*ALOG((X + BM)/1.66943)
     A  - ALOG((X**2 - X*BM + BM**2)/0.77871))
	TERM3 = 7.53670*(ATAN((2.0*X - BM)/1.15949) - 0.85399)
      PSI_M = TERM1 + TERM2 + TERM3
      RETURN
C
C	  Neutral stratification.
30    PSI_M = 0.0
      RETURN
C
      END
C
C
C
      FUNCTION PSI_H(ZS,L)
C  Last modification 11/20/08.
C  This function computes the stability correction for either the
C  temperature or humidity profile based on new Psi functions reported
C  by Grachev et al. (2007) for stable stratification.
C  ZS is the height of either the temperature or humidity observation.
C  L is the Obukhov length.
C
      REAL L
C
      ZETA = ZS/L						!Stability parameter
      IF(ZETA) 10, 30, 20				!Check stability
C
C	  Unstable stratification (Paulson, 1970).
10    X = (1 - 16.0*ZETA)**0.25
      PSI_H = 2.0*ALOG(0.5*(1 + X*X))
      RETURN
C
C	  Stable stratification (Grachev et al. 2007).
20    TERM1 = -2.5*ALOG(1.0 + 3.0*ZETA + ZETA**2)
      TERM2 = 1.11803*(ALOG((ZETA + 0.38197)/(ZETA + 2.61803))
     A + 1.92485)
      PSI_H = TERM1 + TERM2
      RETURN
C
C	  Neutral stratification.
30    PSI_H = 0.0
      RETURN
C
      END
C
C
C
      SUBROUTINE ANDREAS_87(USTAR,Z0,RSTAR,ZT,ZQ)
C  Last modification 12/2/08.
C  This subroutine computes the scalar roughnesses for temperature
C  (ZT) and humidity (ZQ) over now using the Andreas (1987)
C  theoretical model.
C  USTAR is the friction velocity
C  Z0 is the momentum roughness length.
C  RSTAR is the roughness Reynolds number.
C  All units are mks.
C  The predictive equation is
C
C	  ln(ZS/Z0)  =  B0  +  B1*ln(RSTAR)  +  B2*(ln(RSTAR)**2)
C
C
      REAL LNZSZ0, LNRSTAR
C	  Natural logs or ZS/Z0 and RSTAR.
C
      DIMENSION RS(3), BT(3,3), BQ(3,3)
C	  RS delimits the RSTAR ranges
C	  The B's are coefficients in the second-order predictive equation.
C
C
      DATA RS/0.135,2.5,1000./,
     A BT/1.250,0.0,0.0,0.149,-0.550,0.0,0.317,-0.565,-0.183/,
     B BQ/1.610,0.0,0.0,0.351,-0.628,0.0,0.396,-0.512,-0.180/
C	  RS is the limits of the RSTAR ranges.
C	  BT and BQ are the coefficients of the predictive equation
C	  in these various RSTAR ranges.
      DATA ZS_LIM/7.0E-8/
C	  Do not let zT or zQ get smaller than mean free path of air
C	  molecules (in meters).
C
C
C	  Find RSTAR range.
      DO 100 IND_RS=1,3
      IF(RSTAR .LE. RS(IND_RS)) GO TO 110
C	  Otherwise, look in next higher range.
100   CONTINUE
C
C	  If exit to this statement, RSTAR is beyond range of the model.
      PRINT 700, RSTAR
700   FORMAT(/5X,'R* = ',1PF7.1,' is beyond the snow model''s limit',
     A ' of 1000.')
      PAUSE 'Type CONTINUE to continue.'
      IND_RS = 3
C
C	  Compute ZT and ZQ.
110   LNRSTAR = ALOG(RSTAR)
C	  First, for temperature.
      LNZSZ0 = BT(1,IND_RS) + BT(2,IND_RS)*LNRSTAR
     A + BT(3,IND_RS)*(LNRSTAR**2)
      ZT = Z0*EXP(LNZSZ0)				!Temperature roughness
      IF(ZT .LT. ZS_LIM) ZT = ZS_LIM	!Lower limit to zT
C
C	  Next, for humidity.
      LNZSZ0 = BQ(1,IND_RS) + BQ(2,IND_RS)*LNRSTAR
     A + BQ(3,IND_RS)*(LNRSTAR**2)
      ZQ = Z0*EXP(LNZSZ0)				!Humidity roughness
      IF(ZQ .LT. ZS_LIM) ZQ = ZS_LIM	!Lower limit of zQ
C
      RETURN
      END
C
C
C
      SUBROUTINE BULK_COEFS(Z0,ZT,ZQ,L,RU,RT,RQ,CD,CH,CE,KODE)
C  Last modification 7/17/14.
C  This subroutine calculates the stability-corrected transfer coefficients
C  for momentum (CD), sensible heat (CH), and latent heat (CE) appropriate
C  for reference heights ZU, ZT, and ZQ, respectively.
C  The input variables are the Obukhov length (L) and the roughness
C  lengths for momentum (Z0), temperature (ZT), and humidity (ZQ).
C  KODE = 0 if desire calculations for neutral stability.
C
C
      USE CONSTANTS
C
      REAL L
C
C
      C(Z1,Z2,Z01,Z02,PSI1,PSI2) = (K*K)/((ALOG(Z1/Z01) - PSI1)
     A *(ALOG(Z2/Z02) - PSI2))
C	  Generic computation of bulk transfer coefficient.
C
C
      IF(KODE .NE. 0) GO TO 50		!For stability-dependent cases
C	  Otherwise, for neutral stratification.
      PSIM = 0.0
      PSIHT = 0.0
      PSIHQ = 0.0
      GO TO 200
C
C
50    IF(L .NE. 0.0) GO TO 100
C	  Otherwise, would be dividing by zero.
      PAUSE '  Obukhov length L is specified as zero.  Wrong!'
C
C	  Normal processing for both stable and unstable stratification.
100   PSIM = PSI_M(RU,L)				!Computes phim
      PSIHT = PSI_H(RT,L)				!Computes phih for temperature
      PSIHQ = PSI_H(RQ,L)				!Computes phih for humidity
C
C	  Compute transfer coefficients.
200   CD = C(RU,RU,Z0,Z0,PSIM,PSIM)	!Drag coefficient
      CH = C(RU,RT,Z0,ZT,PSIM,PSIHT)	!Transfer coef. for sensible heat
      CE = C(RU,RQ,Z0,ZQ,PSIM,PSIHQ)	!Transfer coef. for latent heat
C
      RETURN
      END
C
C
C
      FUNCTION SPEED(UR,USTAR,ZI,L)
C  Last modification 10/22/14.
C  This function computes a gustiness, G, to add to the wind speed (UR) in
C  light winds to maintain fluxes in nearly calm conditions.
C  We use the COARE gust factor BETA*WSTAR (Fairall et al., 1996) in
C  unstable conditions and an adaptation of the windless coefficient of Jordan
C  et al. (1999) in stable conditions that goes to approximately zero for wind
C  speeds above 3 m/s.
C  (1999) in stable conditions.
C  USTAR is the friction velocity.
C  ZI is the height of the inversion base.
C  L is the Obukhov length.
C
C
      USE CONSTANTS
C
      REAL L
C
      DATA W0/0.5/, BETA/1.25/
C	  W0 is the windless coefficient in m/s.
C	  BETA is an empirical coefficient in the COARE gustiness relation.
C
C
      IF(ABS(L) .LT. 1000.0) GO TO 50
C	  Otherwise, neutral stability for practical purposes.
      SPEED = UR
      RETURN
C
C	  Check stratification.
50    IF(L) 70, 80, 80
C
C	  COARE algorithm's gustiness correction.
70    WSTAR = USTAR*((-ZI/(K*L))**0.333333)
      SPEED = SQRT((UR**2) + ((BETA*WSTAR)**2))
      RETURN
C
C	  Modified windless coefficient.
80    SPEED = UR + W0/COSH(UR)
      RETURN
C
      END
C
C
C
      SUBROUTINE FLUXES(USTAR,TSTAR,QSTAR,RHOA,Q,CP,LV,TAU,HS,HL,EVAP)
C  Last modification 12/2/08.
C  This subroutine computes the interfacial surface fluxes of momentum
C  (TAU) and sensible (HS) and latent heat (HL).
C
C
      REAL LV
C
      DATA RHOW/1000.0/
C	  Density of pure water.
C
C
      TAU = RHOA*(USTAR**2)			!Momentum flux
      HS = -RHOA*CP*(1.0 + 0.85*Q)*USTAR*TSTAR
									!Sensible heat flux
      HL = -RHOA*LV*USTAR*QSTAR		!Latent heat flux
C	  Heat fluxes are in W/m**2.  TAU is in N/m**2.
C
C	  Compute evaporation rate in water equivalent.
      EVAP = HL/(LV*RHOW)				!In m/s
      EVAP = 1000.*3600.*24.*EVAP		!In mm/day
C
C
      RETURN
      END
