C     PROGRAM TEST_SHEBA_BFA
C  Last modification 7/22/14.
C  Version 2.0.
C  This program is an interactive interface with the SHEBA bulk flux
C  algorithm developed by the SHEBA Atmospheric Surface Flux Group.
C  It allows standalone calculations of the turbulent surface fluxes
C  for a given set of input conditions for compact winter sea ice, for summer
C  sea ice with melt ponds and leads, and for the mariginal ice zone.
C  This new version of the SHEBA bulk flux algorithm (Version 2.0) adds
C  another option for parameterizing the aerodynamic roughness, Z0.
C  Following Andreas (2011), this version allows specifying the physical
C  roughness, XI, which the Andreas (2011) algorithm then relates to Z0.
C  The original option of obtaining aerodynamic roughness from the
C  ice concentration is still available.
C  With this option, the only way to designate summer or the marginal ice
C  zone is through the input ice concentration.  If the ice concentration
C  is less than 99.5%, the program assumes that the water fraction occurs
C  as melt ponds and leads during summer or the open water surrounding the
C  broken ice in the marginal ice zone.
C  Use this test program to see how the calls work.  Then you can implement
C  the subroutines in your own, larger program.
C  This program asks for meteorological variables and then iteratively
C  computes the surface stress (momentum flux) and the sensible and
C  latent heat fluxes.
C  The wind speed also includes the COARE gustiness factor (Fairall et al. 1996) in
C  unstable stratification and a modified windless coefficient for stable stratification
C  based on ideas in Jordan et al. (1999).
C
C
      USE CONSTANTS                   !Specifies K and G
	USE CONDITIONS                  !For passing input conditions
	USE RESULTS                     !For passing results
C
      IMPLICIT NONE
C
C
      INTEGER IANS
C
C       Fluxes.
      REAL*4 TAU, HS, HL, EVAP
C	  TAU is the momentum flux.
C       HS is the sensible heat flux.
C       HL is the latent heat flux.
C       EVAP is the evporative flux associated with HL.
C
C       External.
      EXTERNAL SHEBA_BULK
C
C
C	  Ask for meteorological data.
10    PRINT 700
700   FORMAT(/5X,'Welcome to the SHEBA Bulk Flux Algorithm, V 2.0.'//
     A5X,'What is the wind speed (in m/s) and the height (in m)',
     B ' of the observation?')
      READ(*,*) UR, RU
C
      PRINT 705
705   FORMAT(5X,'What is the air temperature (in deg C)'
     A ' and its height?')
      READ(*,*) TR, RT
C
      PRINT 710
710   FORMAT(5X,'What is the relative humidity (in % with respect',
     A ' to water)?')
      READ(*,*) RH
	RQ = RT
C       Humidity height RQ must be the same as RT because, later, compute
C       specific humidity from relative humidity and temperature.
C       That is, they must be collocated.
C
      PRINT 715
715   FORMAT(5X,'What is the surface temperature (in deg C)'/
     A 5X,'and the barometric pressure (in mb)?')
      READ(*,*) TS, P
C
15    PRINT 720
720   FORMAT(5X,'Do you know the physical roughness (1) (xi parameter'
     A ' of Banke et al. 1980)'/
     B 5X,'or do you want to determine roughness from the ice'
     C ' concentration (2)?')
      READ(*,*) CHOICE
	IF(CHOICE .LT. 1 .OR. CHOICE .GT. 2) GO TO 15
C        Bad answer if true.
C
C        Choose how to obtain aerodynamic roughness.
      SET_ROUGHNESS: SELECT CASE (CHOICE)
	CASE (1)                        !Get XI
	CONC = 1.0                      !Set to 100% if specify XI
20    PRINT 725
725   FORMAT(5X,'Enter xi in centimeters.')
      READ(*,*) XI
	IF(XI .LT. 0.0) GO TO 20        !Bad value if true
C
      CASE (2)                        !Get ice concentration
	XI = 0.0                        !Set to 0 is specify ice concentration
25    PRINT 730
730   FORMAT(5X,'What is the ice concentration in percent?'/
     A 5X,'A value less than 99.5% invokes the algorithm for summer '
     B 'sea ice'/,5X,'and the marginal ice zone.')
      READ(*,*) CONC
	IF(CONC .GT. 100.0 .OR. CONC .LT. 0.0) GO TO 25
C		Otherwise, good value.
      CONC = CONC/100.0				!Convert to fractional coverage
	END SELECT SET_ROUGHNESS
C
C
      CALL SHEBA_BULK(TAU,HS,HL,EVAP)
C	  Returns all the fluxes.  Additional calculations are in COMMON /RESULTS/.
C
C	  Print results.
      PRINT 900, UR, RU, TR, RT, QR, RQ, RH_I, P, TS, CONC
900   FORMAT(/5X,'Wind speed = ',F5.2,' m/s at ',F5.2,' meters'/
     A 5X,'Temperature = ',F6.2,' deg C at ',F5.2,' meters'/
     B 5X,'Specific humidity = ',1PE9.3,' kg/kg at ',0PF5.2,
     C ' meters'/
     D 5X,'Relative humidity with respect to ice = ',F6.2,'%'/
     D 5X,'Pressure = ',F6.1,' mb'/
     E 5X,'Surface temperature = ',F7.3,' deg C'/
     F 5X,'Ice concentration = ',2PF6.2,'%')
C
      ROUGHNESS_MODEL: SELECT CASE (CHOICE)
	CASE (1)                        !Have specified XI
	PRINT 910, XI
910   FORMAT(5X,'Aerodynamic roughness model is based on physical'
     A ' roughness'/10X,'with xi = ',0PF6.3,' cm'/)
      CASE (2)                        !Have specified ice concentration
	PRINT 920
920   FORMAT(5X,'Aerodynamic roughness model is based on ice'
     A ' concentration.'/
     B 5X,'Concentrations less than 99.5% invoke the summer model.')
      END SELECT ROUGHNESS_MODEL
C
      PRINT 930, ITS, USTAR, TSTAR, QSTAR, L, WIND
930   FORMAT(5X,'Iterations = ',I2/
     A 5X,'u* = ',0PF7.4,' m/s'/
     B 5X,'t* = ',F8.5,' deg C'/
     C 5X,'q* = ',1PE10.3,' kg/kg'/
     D 5X,'Obukhov length = ',0PF9.3,' m'/
     E 5X,'Effective wind = ',F8.3,' m/s')
C
      PRINT 940, Z0, ZT, ZQ, RSTAR
940   FORMAT(5X,'Z0 = ',1PE9.3,' m'/
     A 5X,'ZT = ',1PE9.3,' m'/
     B 5X,'ZQ = ',1PE9.3,' m'/
     C 5X,'R* = ',0PF8.3)
C
      PRINT 950, TAU, HS, HL, EVAP
950   FORMAT(5X,'Surface Stress = ',1PE9.3,' N/m**2'/
     A 5X,'Sensible Heat Flux = ',0PF10.3,' W/m**2'/
     B 5X,'Latent Heat Flux = ',F10.3,' W/m**2'/
     C 5X,'Evaporation Rate = ',F8.3,' mm/day')
C
C
400   TYPE *, '  Enter new conditions (1) or end (0)?'
      READ(*,*) IANS
      IF(IANS .EQ. 1) GO TO 10		!Go again
      IF(IANS .NE. 0) GO TO 400		!Get a better answer
C
      END